/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4.implementation;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools4.DroolsInterface;
import org.drools.RuleBase;
import org.drools.StatefulSession;
import org.drools.StatelessSession;

/**
 * 
 * @author mbo
 *
 */
abstract class DroolsInterfaceImplementation<T> implements DroolsInterface{
	
	private final Logger logger = Logger.getLogger(this.getClass());
	
	private Class<T> clazz; 
		
	private RuleBase ruleBase;
	
	private StatefulSession session;

	@Override
	public RuleBase getRuleBase() {
		return ruleBase;
	}
	
	protected DroolsInterfaceImplementation(Class<T> clazz,RuleBase ruleBase) {
		this.clazz=clazz;
		this.ruleBase=ruleBase;
	}

	public Class<T> getClazz() {
		return clazz;
	}

	@Override
	public StatefulSession getCurrentSession() {
		return session;
	}
	
	void setCurrentSession(StatefulSession session) {
		this.session=session;
	}
	
	Iterator<?> runStatelessSession(List<Object> fact) {
		StatelessSession dss = getRuleBase().newStatelessSession();
		new DroolsWorkingMemoryLogger(dss);
		return dss.executeWithResults(fact).iterateObjects();
	}
	
	Iterator<?> runStatefulSession(List<Object> fact,boolean requiredExisting) {
		if (requiredExisting && session==null)  {
			String msg = "No session found when required";
			logger.error(msg);
			throw new IllegalArgumentException(msg);
		}
		if (session==null) {
			session=ruleBase.newStatefulSession();
			new DroolsWorkingMemoryLogger(session);
		}
		for(Object o:fact) session.insert(o);
		session.fireAllRules();
		return session.iterateObjects();
	}
	
	Object getResult(Class<?> returnType,Iterator<?> result) {
		if (returnType.equals(Void.TYPE)) return null;			
		if (result!=null) {
			if (returnType.isArray()) {
				Class<?> real = returnType.getComponentType();
				List<Object> lst = new ArrayList<Object>();
				Iterator<?> i = result;
				while(i.hasNext()) {
					Object o = i.next();
					if (real.isAssignableFrom(o.getClass())) lst.add(o);
				}
				return lst.toArray((Object[])Array.newInstance(real,lst.size()));
			} else {
				Iterator<?> i = result;
				while(i.hasNext()) {
					Object o = i.next();
					if (returnType.isAssignableFrom(o.getClass())) return o;
				}
			}
		}
		return null;
	}

}