/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4.implementation;

import java.lang.annotation.Annotation;
import java.lang.reflect.Proxy;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools4.DroolsInterface;
import org.boretti.drools.integration.drools4.DroolsServiceType;
import org.boretti.drools.integration.drools4.annotations.DroolsService;
import org.boretti.drools.integration.drools4.exceptions.NotAnInterfaceDroolsError;
import org.boretti.drools.integration.drools4.exceptions.NotAnnotatedInterfaceDroolsError;
import org.boretti.drools.integration.drools4.implementation.DroolsInterfaceImplementation;
import org.boretti.drools.integration.drools4.implementation.DroolsCompiledInterfaceImplementation;

public class DroolsProviderImplementation {
	
	private final Logger logger = Logger.getLogger(this.getClass());
	
	private DroolsProviderImplementation() {}
	
	private static final DroolsProviderImplementation instance = new DroolsProviderImplementation();
	
	public static final DroolsProviderImplementation getInstance() {
		return instance;
	}
	
	@SuppressWarnings("unchecked")
	public <T> T getService(Class<T> clazz) {
		if (logger.isDebugEnabled()) logger.debug("Gettting service for "+clazz);
		if (clazz==null) {
			String msg = "Received class is null";
			logger.error(msg);
			throw new IllegalArgumentException(msg);
		}
		if (!clazz.isInterface()) {
			String msg = "Not an interface "+clazz;
			logger.error(msg);
			throw new NotAnInterfaceDroolsError(msg);
		}
		Annotation as[] = clazz.getAnnotations();
		boolean valid=false;
		DroolsServiceType dst = null;
		String resource = null;
		for(Annotation a:as) {
			if (a.annotationType().equals(DroolsService.class)) {
				valid=true;
				DroolsService ds = (DroolsService)a;
				dst=ds.type();
				resource = ds.resourceName();
				break;
			}
		}
		if(!valid) {
			String msg = "Invalid interface (missing annotation) "+clazz;
			logger.error(msg);
			throw new NotAnnotatedInterfaceDroolsError(msg);
		}
		DroolsInterfaceImplementation<T> inter=null;
		if (dst==null) {
			String msg = "Type must not be null";
			logger.error(msg);
			throw new IllegalArgumentException(msg);
		}
		if (DroolsServiceType.COMPILED.equals(dst)) {
			if (resource==null || resource.trim().equalsIgnoreCase("")) resource = clazz.getName()+".cdrl";
			 inter = new DroolsCompiledInterfaceImplementation<T>(clazz, resource);
		} else if (DroolsServiceType.SOURCE.equals(dst)) {
			if (resource==null || resource.trim().equalsIgnoreCase("")) resource = clazz.getName()+".drl";
			inter = new DroolsSourceInterfaceImplementation<T>(clazz,resource);
		}
		if (logger.isDebugEnabled()) logger.debug("Creating proxy using "+clazz+" from "+dst+" using resource "+resource);
		return (T) Proxy.newProxyInstance(clazz.getClassLoader(),new Class[] {clazz,DroolsInterface.class},new DroolsProxy<T>(clazz,inter));
	}
}
